"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CosmwasmPlatform = void 0;
const cosmwasm_stargate_1 = require("@cosmjs/cosmwasm-stargate");
const stargate_1 = require("@cosmjs/stargate");
const connect_sdk_1 = require("@wormhole-foundation/connect-sdk");
const chain_1 = require("./chain");
const constants_1 = require("./constants");
const contracts_1 = require("./contracts");
const gateway_1 = require("./gateway");
const platformUtils_1 = require("./platformUtils");
const ibc_1 = require("./protocols/ibc");
const tokenBridge_1 = require("./protocols/tokenBridge");
var _ = CosmwasmPlatform;
/**
 * @category Cosmwasm
 */
var CosmwasmPlatform;
(function (CosmwasmPlatform) {
    CosmwasmPlatform.platform = "Cosmwasm";
    CosmwasmPlatform.network = connect_sdk_1.DEFAULT_NETWORK;
    CosmwasmPlatform.conf = (0, connect_sdk_1.networkPlatformConfigs)(CosmwasmPlatform.network, CosmwasmPlatform.platform);
    CosmwasmPlatform.contracts = new contracts_1.CosmwasmContracts(CosmwasmPlatform.conf);
    // TODO: re-export all
    CosmwasmPlatform.nativeTokenId = platformUtils_1.CosmwasmUtils.nativeTokenId, CosmwasmPlatform.isNativeTokenId = platformUtils_1.CosmwasmUtils.isNativeTokenId, CosmwasmPlatform.isNativeDenom = platformUtils_1.CosmwasmUtils.isNativeDenom, CosmwasmPlatform.isSupportedChain = platformUtils_1.CosmwasmUtils.isSupportedChain, CosmwasmPlatform.getDecimals = platformUtils_1.CosmwasmUtils.getDecimals, CosmwasmPlatform.getBalance = platformUtils_1.CosmwasmUtils.getBalance, CosmwasmPlatform.getBalances = platformUtils_1.CosmwasmUtils.getBalances, CosmwasmPlatform.sendWait = platformUtils_1.CosmwasmUtils.sendWait, CosmwasmPlatform.getCurrentBlock = platformUtils_1.CosmwasmUtils.getCurrentBlock, CosmwasmPlatform.chainFromChainId = platformUtils_1.CosmwasmUtils.chainFromChainId, CosmwasmPlatform.chainFromRpc = platformUtils_1.CosmwasmUtils.chainFromRpc;
    CosmwasmPlatform.getGatewayRpc = gateway_1.Gateway.getRpc, CosmwasmPlatform.getGatewayWrappedAsset = gateway_1.Gateway.getWrappedAsset, CosmwasmPlatform.gatewayAddress = gateway_1.Gateway.gatewayAddress, CosmwasmPlatform.getGatewaySourceChannel = gateway_1.Gateway.getGatewaySourceChannel;
    function setConfig(_network, _conf) {
        CosmwasmPlatform.conf = _conf ? _conf : (0, connect_sdk_1.networkPlatformConfigs)(CosmwasmPlatform.network, CosmwasmPlatform.platform);
        CosmwasmPlatform.contracts = new contracts_1.CosmwasmContracts(CosmwasmPlatform.conf);
        CosmwasmPlatform.network = _network;
        return CosmwasmPlatform;
    }
    CosmwasmPlatform.setConfig = setConfig;
    async function getRpc(chain) {
        const rpcAddress = CosmwasmPlatform.conf[chain].rpc;
        return await cosmwasm_stargate_1.CosmWasmClient.connect(rpcAddress);
    }
    CosmwasmPlatform.getRpc = getRpc;
    function getChain(chain) {
        if (chain in CosmwasmPlatform.conf)
            return new chain_1.CosmwasmChain(CosmwasmPlatform.conf[chain]);
        throw new Error("No configuration available for chain: " + chain);
    }
    CosmwasmPlatform.getChain = getChain;
    // TODO: should other platforms have something like this?
    function getNativeDenom(chain) {
        // TODO: required because of const map
        if (CosmwasmPlatform.network === "Devnet")
            throw new Error("No devnet native denoms");
        return (0, constants_1.chainToNativeDenoms)(CosmwasmPlatform.network, chain);
    }
    CosmwasmPlatform.getNativeDenom = getNativeDenom;
    async function parseTransaction(chain, rpc, txid) {
        const tx = await rpc.getTx(txid);
        if (!tx)
            throw new Error("No Transaction found: " + txid);
        return [gateway_1.Gateway.parseWormholeMessage(tx)];
    }
    CosmwasmPlatform.parseTransaction = parseTransaction;
    async function getTokenBridge(rpc) {
        return await tokenBridge_1.CosmwasmTokenBridge.fromProvider(rpc, CosmwasmPlatform.contracts);
    }
    CosmwasmPlatform.getTokenBridge = getTokenBridge;
    async function getIbcBridge(rpc) {
        return await ibc_1.CosmwasmIbcBridge.fromProvider(rpc, CosmwasmPlatform.contracts);
    }
    CosmwasmPlatform.getIbcBridge = getIbcBridge;
    CosmwasmPlatform.getQueryClient = (rpc) => {
        // @ts-ignore
        const tmClient = rpc.getTmClient();
        return stargate_1.QueryClient.withExtensions(tmClient, stargate_1.setupBankExtension, stargate_1.setupIbcExtension);
    };
    // cached channels from config if available
    CosmwasmPlatform.getIbcChannels = (chain) => {
        return constants_1.networkChainToChannels.has(CosmwasmPlatform.network, chain)
            ? constants_1.networkChainToChannels.get(CosmwasmPlatform.network, chain)
            : null;
    };
})(CosmwasmPlatform || (exports.CosmwasmPlatform = CosmwasmPlatform = {}));
//# sourceMappingURL=platform.js.map